@@ -1,11 +1,10 @@
 # This file is part of xrootdpyfs
-# Copyright (C) 2015 CERN.
+# Copyright (C) 2015-2020 CERN.
 #
 # xrootdpyfs is free software; you can redistribute it and/or modify it under
 # the terms of the Revised BSD License; see LICENSE file for more details.
 
 pydocstyle xrootdpyfs && \
-sphinx-build -qnN docs docs/_build/html && \
-python setup.py test && \
-sphinx-build -qnNW -b doctest docs docs/_build/doctest && \
-isort -rc -c tests/ xrootdpyfs/
+isort -rc -c -df . && \
+sphinx-build -qnNW docs docs/_build/html && \
+pytest tests/
