@@ -11,16 +11,15 @@
 from __future__ import absolute_import, print_function
 
 import pytest
-from fs.errors import ResourceNotFoundError
-
 from conftest import mkurl
+from fs.errors import ResourceNotFoundError
 from xrootdpyfs import XRootDPyFS
 
 
 def test_getcontents(tmppath):
     """Test getcontents."""
     fs = XRootDPyFS(mkurl(tmppath))
-    assert fs.getcontents('data/testa.txt') == "testa.txt\n"
+    assert fs.getcontents('data/testa.txt') == b"testa.txt\n"
     pytest.raises(ResourceNotFoundError, fs.getcontents, 'data/invalid.txt')
 
 
@@ -28,4 +27,4 @@ def test_setcontents(tmppath):
     """Test setcontents."""
     fs = XRootDPyFS(mkurl(tmppath))
     fs.setcontents('data/testa.txt', "mytest")
-    assert fs.getcontents('data/testa.txt') == "mytest"
+    assert fs.getcontents('data/testa.txt') == b"mytest"
