@@ -48,10 +48,10 @@ class XRootDPyFile(object):
        non-streamed reading/writing for XRootD.
 
     :param path: Path to file that should be opened.
-    :type path: string
+    :type path: str
     :param mode: Mode of file to open, identical to the mode string used
         in 'file' and 'open' builtins.
-    :type mode: string
+    :type mode: str
     :param buffering: An optional integer used to set the buffering policy.
         Pass 0 to switch buffering off (only allowed in binary mode),
         1 to select line buffering (only usable in text mode), and
@@ -165,8 +165,8 @@ class XRootDPyFile(object):
         """Exit context manager method."""
         self.close()
 
-    def next(self):
-        """Return next item for file iteration."""
+    def __next__(self):
+        """Return next item for file iteration for Python 3."""
         item = self._next_func(*self._next_args[0], **self._next_args[1])
         if not item:
             raise StopIteration
@@ -178,14 +178,22 @@ class XRootDPyFile(object):
         If no ``sizehint`` is provided the entire file is read! Multiple calls
         to this method after EOF as been reached, will return an empty string.
 
-        :oaram sizehint: Number of bytes to read from file object.
+        :param sizehint: Number of bytes to read from file object.
         """
         if self.closed:
             raise ValueError("I/O operation on closed file.")
 
         self._assert_mode("r-")
 
-        chunksize = sizehint if sizehint > 0 else self.size
+        chunksize = sizehint if sizehint > 0 else self.size - self._ipp
+
+        if chunksize >= 2147483648:  # 2GB in bytes
+                raise IOError(
+                    "Chunksize is set to %s which is more than 2GB."
+                    "This is not supported!" % chunksize
+                )
+        elif chunksize < 0:
+            chunksize = 1
 
         # Read data
         statmsg, res = self._file.read(
